@@ -1,3 +1,18 @@
+
+python_has_modules <- function(python, modules) {
+
+  # write code to tempfile
+  file <- tempfile("reticulate-python-", fileext = ".py")
+  code <- paste("import", modules)
+  writeLines(code, con = file)
+  on.exit(unlink(file), add = TRUE)
+
+  # invoke Python
+  status <- system2(python, shQuote(file), stdout = FALSE, stderr = FALSE)
+  status == 0L
+
+}
+
 python_has_module <- function(python, module) {
   code <- paste("import", module)
   args <- c("-E", "-c", shQuote(code))
@@ -8,23 +23,15 @@ python_has_module <- function(python, module) {
 python_version <- function(python) {
   code <- "import platform; print(platform.python_version())"
   args <- c("-E", "-c", shQuote(code))
-  output <- system2(python, args, stdout = TRUE)
-  numeric_version(output)
+  output <- system2(python, args, stdout = TRUE, stderr = FALSE)
+  sanitized <- gsub("[^0-9.-]", "", output)
+  numeric_version(sanitized)
 }
 
 python_module_version <- function(python, module) {
   fmt <- "import %1$s; print(%1$s.__version__)"
   code <- sprintf(fmt, module)
   args <- c("-E", "-c", shQuote(code))
-  output <- system2(python, args, stdout = TRUE)
+  output <- system2(python, args, stdout = TRUE, stderr = FALSE)
   numeric_version(output)
 }
-
-python_unix_binary <- function(bin) {
-  locations <- file.path(c("/usr/bin", "/usr/local/bin", path.expand("~/.local/bin")), bin)
-  locations <- locations[file.exists(locations)]
-  if (length(locations) > 0)
-    locations[[1]]
-  else
-    NULL
-}
