@@ -62,7 +62,19 @@ import <- function(module, as = NULL, convert = TRUE, delay_load = FALSE) {
     ensure_python_initialized(required_module = module)
 
     # import the module
-    py_module_import(module, convert = convert)
+    hookName <- paste("reticulate", module, "load", sep = "::")
+    module <- py_module_import(module, convert = convert)
+
+    # run load hooks
+    hooks <- getHook(hookName)
+    for (hook in hooks)
+      tryCatch(hook(module), error = warning)
+
+    # remove hooks (we only want to run on first import)
+    setHook(hookName, list(), "replace")
+
+    # return imported module
+    module
   }
 
   # delay load case (wait until first access)
@@ -237,27 +249,36 @@ py_has_convert <- function(x) {
 }
 
 py_maybe_convert <- function(x, convert) {
-  if (convert || py_is_callable(x)) {
 
-    # capture previous convert for attr
-    attrib_convert <- py_has_convert(x)
+  # if this is already an R object, nothing to do
+  if (!inherits(x, "python.builtin.object"))
+    return(x)
 
-    # temporarily change convert so we can call py_to_r and get S3 dispatch
-    envir <- as.environment(x)
-    assign("convert", convert, envir = envir)
-    on.exit(assign("convert", attrib_convert, envir = envir), add = TRUE)
+  # if it's neither convertable nor callable,
+  # nothing to do
+  convertable <- convert || py_is_callable(x)
+  if (!convertable)
+    return(x)
 
-    # call py_to_r
-    x <- py_to_r(x)
-  }
+  # perform conversion
+  # capture previous convert for attr
+  attrib_convert <- py_has_convert(x)
+
+  # temporarily change convert so we can call py_to_r and get S3 dispatch
+  envir <- as.environment(x)
+  assign("convert", convert, envir = envir)
+  on.exit(assign("convert", attrib_convert, envir = envir), add = TRUE)
+
+  # call py_to_r
+  py_to_r(x)
 
-  x
 }
 
 # helper function for accessing attributes or items from a
 # Python object, after validating that we do indeed have
 # a valid Python object reference
 py_get_attr_or_item <- function(x, name, prefer_attr) {
+
   # resolve module proxies
   if (py_is_module_proxy(x))
     py_resolve_module_proxy(x)
@@ -286,16 +307,27 @@ py_get_attr_or_item <- function(x, name, prefer_attr) {
   }
 
   # get the attrib and convert as needed
+  object <- NULL
   if (prefer_attr) {
     object <- py_get_attr(x, name)
   } else {
 
     # if we have an attribute, attempt to get the item
-    # but allow for fallback to that attribute
+    # but allow for fallback to that attribute. note that
+    # the logic here is fairly convoluted but is necessary
+    # to maintain backwards compatibility with a number of
+    # CRAN packages (hopefully we can simplify this in the
+    # future)
     if (py_has_attr(x, name)) {
-      object <- py_get_item(x, name, silent = TRUE)
+
+      # try to get item
+      if (py_has_attr(x, "__getitem__"))
+        object <- py_get_item(x, name, silent = TRUE)
+
+      # fallback to attribute
       if (is.null(object))
         object <- py_get_attr(x, name)
+
     } else {
       # we don't have an attribute; only attempt item
       # access and allow normal error propagation
@@ -339,10 +371,6 @@ as.environment.python.builtin.object <- function(x) {
 }
 
 
-#' @export
-`[[.python.builtin.object` <- `$.python.builtin.object`
-
-
 #' @export
 `$<-.python.builtin.object` <- function(x, name, value) {
   if (!py_is_null_xptr(x) && py_available())
@@ -1157,6 +1185,17 @@ py_capture_output <- function(expr, type = c("stdout", "stderr")) {
   output
 }
 
+py_flush_output <- function() {
+
+  if (!is_python3())
+    return()
+
+  sys <- import("sys", convert = TRUE)
+  sys$stdout$flush()
+  sys$stderr$flush()
+
+}
+
 
 
 
@@ -1179,6 +1218,7 @@ py_capture_output <- function(expr, type = c("stdout", "stderr")) {
 #' @export
 py_run_string <- function(code, local = FALSE, convert = TRUE) {
   ensure_python_initialized()
+  on.exit(py_flush_output(), add = TRUE)
   invisible(py_run_string_impl(code, local, convert))
 }
 
@@ -1186,6 +1226,7 @@ py_run_string <- function(code, local = FALSE, convert = TRUE) {
 #' @export
 py_run_file <- function(file, local = FALSE, convert = TRUE) {
   ensure_python_initialized()
+  on.exit(py_flush_output(), add = TRUE)
   invisible(py_run_file_impl(file, local, convert))
 }
 
