@@ -15,17 +15,23 @@
 #'   environment within `virtualenv_root()`. When `NULL`, the virtual environment
 #'   as specified by the `RETICULATE_PYTHON_ENV` environment variable will be
 #'   used instead.
+#'
 #' @param packages A character vector with package names to install or remove.
+#'
 #' @param ignore_installed Boolean; ignore previously-installed versions of the
 #'   requested packages? (This should normally be `TRUE`, so that pre-installed
 #'   packages available in the site libraries are ignored and hence packages
 #'   are installed into the virtual environment.)
+#'
 #' @param confirm Boolean; confirm before removing packages or virtual
 #'   environments?
+#'
 #' @param python The path to a Python interpreter, to be used with the created
 #'   virtual environment. When `NULL`, the Python interpreter associated with
 #'   the current session will be used.
 #'
+#' @param ... Optional arguments; currently ignored for future expansion.
+#'
 #' @name virtualenv-tools
 NULL
 
@@ -45,6 +51,7 @@ virtualenv_list <- function() {
 #' @rdname virtualenv-tools
 #' @export
 virtualenv_create <- function(envname = NULL, python = NULL) {
+
   path <- virtualenv_path(envname)
   name <- if (is.null(envname)) path else envname
 
@@ -54,15 +61,12 @@ virtualenv_create <- function(envname = NULL, python = NULL) {
     return(invisible(path))
   }
 
-  # if the user hasn't requested a particular Python binary,
-  # infer from the currently active Python session
   python <- virtualenv_default_python(python)
+  module <- virtualenv_module(python)
+
   writeLines(paste("Creating virtual environment", shQuote(name), "..."))
   writeLines(paste("Using python:", python))
 
-  # choose appropriate tool for creating virtualenv
-  module <- virtualenv_module(python)
-
   # use it to create the virtual environment
   args <- c("-m", module, "--system-site-packages", path.expand(path))
   result <- system2(python, shQuote(args))
@@ -72,6 +76,11 @@ virtualenv_create <- function(envname = NULL, python = NULL) {
     stop(msg, call. = FALSE)
   }
 
+  # upgrade pip and friends after creating the environment
+  # (since the version bundled with virtualenv / venv may be stale)
+  pip <- virtualenv_pip(path)
+  pip_install(pip, c("pip", "wheel", "setuptools"))
+
   invisible(path)
 }
 
@@ -80,13 +89,23 @@ virtualenv_create <- function(envname = NULL, python = NULL) {
 #' @inheritParams virtualenv-tools
 #' @rdname virtualenv-tools
 #' @export
-virtualenv_install <- function(envname = NULL, packages, ignore_installed = TRUE) {
-
+virtualenv_install <- function(envname = NULL,
+                               packages,
+                               ignore_installed = TRUE,
+                               ...)
+{
+  # create virtual environment on demand
   path <- virtualenv_path(envname)
   if (!file.exists(path))
     path <- virtualenv_create(envname)
 
+  # validate that we've received the path to a virtual environment
   name <- if (is.null(envname)) path else envname
+  if (!is_virtualenv(path)) {
+    fmt <- "'%s' exists but is not a virtual environment"
+    stop(sprintf(fmt, name))
+  }
+
   writeLines(paste("Using virtual environment", shQuote(name), "..."))
 
   # ensure that pip + friends are up-to-date / recent enough
@@ -173,7 +192,11 @@ virtualenv_python <- function(envname = NULL) {
 
 
 virtualenv_exists <- function(envname = NULL) {
-  path <- virtualenv_path(envname)
+
+  # try to resolve path
+  path <- tryCatch(virtualenv_path(envname), error = identity)
+  if (inherits(path, "error"))
+    return(FALSE)
 
   # check that the directory exists
   if (!utils::file_test("-d", path))
@@ -188,38 +211,10 @@ virtualenv_exists <- function(envname = NULL) {
 
 virtualenv_path <- function(envname = NULL) {
 
-  # handle case where envname is NULL (use default / active env)
-  if (is.null(envname)) {
-
-    default <- Sys.getenv("RETICULATE_PYTHON_ENV", unset = NA)
-    if (!is.na(default)) {
-      path <- normalizePath(default, winslash = "/", mustWork = FALSE)
-      if (!is_virtualenv(path)) {
-        fmt <- "there is no virtual environment at path '%s'"
-        stop(sprintf(fmt, path))
-      }
-      return(path)
-    }
-
-    # provide context of caller (if any) when emitting error
-    call <- sys.call(sys.parent())
-    if (is.null(call))
-      call <- sys.call()
-
-    fmt <- "missing environment in call to '%s'"
-    stop(sprintf(fmt, format(sys.call(sys.parent()))), call. = FALSE)
-
-  }
-
-  # treat environment 'names' containing slashes as paths
-  # rather than environments living in WORKON_HOME
-  if (grepl("[/\\]", envname)) {
-    if (file.exists(envname))
-      envname <- normalizePath(envname, winslash = "/")
-    return(envname)
-  }
-
-  file.path(virtualenv_root(), envname)
+  python_environment_resolve(
+    envname = envname,
+    resolve = function(envname) file.path(virtualenv_root(), envname)
+  )
 
 }
 
@@ -231,13 +226,47 @@ virtualenv_pip <- function(envname) {
 
 
 
-virtualenv_default_python <- function(python) {
+virtualenv_default_python <- function(python = NULL) {
 
+  # if the user has supplied a verison of python already, use it
   if (!is.null(python))
     return(python)
 
+  # check for some pre-defined Python sources (prefer Python 3)
+  pythons <- c(
+    Sys.getenv("RETICULATE_PYTHON"),
+    .globals$required_python_version,
+    Sys.which("python3"),
+    Sys.which("python")
+  )
+
+  for (python in pythons) {
+
+    # skip non-existent Python
+    if (!file.exists(python))
+      next
+
+    # get list of required modules
+    version <- tryCatch(python_version(python), error = identity)
+    if (inherits(version, "error"))
+      next
+
+    py2_modules <- c("pip", "virtualenv")
+    py3_modules <- c("pip", "venv")
+    modules <- ifelse(version < 3, py2_modules, py3_modules)
+
+    # ensure these modules are available
+    if (!python_has_modules(python, modules))
+      next
+
+    return(normalizePath(python, winslash = "/"))
+
+  }
+
+  # otherwise, try to explicitly detect Python
   config <- py_discover_config()
   normalizePath(config$python, winslash = "/")
+
 }
 
 
@@ -246,18 +275,19 @@ virtualenv_module <- function(python) {
   py_version <- python_version(python)
 
   # prefer 'venv' for Python 3, but allow for 'virtualenv' for both
+  # (note that 'venv' and 'virtualenv' are largely compatible)
   modules <- "virtualenv"
   if (py_version >= "3.6")
     modules <- c("venv", modules)
 
-  # if we have one of thesem odules available, return it
+  # if we have one of these modules available, return it
   for (module in modules)
     if (python_has_module(python, module))
       return(module)
 
   # virtualenv not available: instruct the user to install
   commands <- new_stack()
-  commands$push("tools for managing Python virtual environments are not installed.")
+  commands$push("Tools for managing Python virtual environments are not installed.")
   commands$push("")
 
   # if we don't have pip, recommend its installation
@@ -266,7 +296,7 @@ virtualenv_module <- function(python) {
     if (python_has_module(python, "easy_install")) {
       commands$push(paste("$", python, "-m easy_install --upgrade --user pip"))
     } else if (is_ubuntu() && dirname(python) == "/usr/bin") {
-      package <- if (py_version < 3) "python-virtualenv" else "python3-venv"
+      package <- if (py_version < 3) "python-pip" else "python3-pip"
       commands$push(paste("$ sudo apt-get install", package))
     } else {
       commands$push("$ curl https://bootstrap.pypa.io/get-pip.py -o get-pip.py")
@@ -275,9 +305,15 @@ virtualenv_module <- function(python) {
     commands$push("")
   }
 
-  # then, recommend installation of virtualenv or ven with pip
+  # then, recommend installation of virtualenv or venv with pip
   commands$push(paste("Install", modules[[1]], "with:"))
-  commands$push(paste("$", python, "-m pip install --upgrade --user", module))
+  if (is_ubuntu() && dirname(python) == "/usr/bin") {
+    package <- if (py_version < 3) "python-virtualenv" else "python3-venv"
+    commands$push(paste("$ sudo apt-get install", package))
+  } else {
+    commands$push(paste("$", python, "-m pip install --upgrade --user", module))
+  }
+  commands$push("\n")
 
   # report to user
   message <- paste(commands$data(), collapse = "\n")
