@@ -33,9 +33,21 @@ typedef long Py_ssize_t;
 #define Py_file_input 257
 #define Py_eval_input 258
 
+#ifdef RETICULATE_PYTHON_DEBUG
+
+#define _PyObject_HEAD_EXTRA            \
+    struct _object *_ob_next;           \
+    struct _object *_ob_prev;
+
+#define _PyObject_EXTRA_INIT 0, 0,
+
+#else
+
 #define _PyObject_HEAD_EXTRA
 #define _PyObject_EXTRA_INIT
 
+#endif /* RETICULATE_PYTHON_DEBUG */
+
 #define PyObject_HEAD  \
 _PyObject_HEAD_EXTRA   \
   Py_ssize_t ob_refcnt; \
@@ -155,7 +167,7 @@ LIBPYTHON_EXTERN PyObject* (*PyImport_Import)(PyObject * name);
 LIBPYTHON_EXTERN PyObject* (*PyImport_GetModuleDict)();
 
 
-LIBPYTHON_EXTERN PyObject* (*PyModule_Create2)(PyModuleDef *def, int);
+LIBPYTHON_EXTERN PyObject* (*PyModule_Create)(PyModuleDef *def, int);
 LIBPYTHON_EXTERN int (*PyImport_AppendInittab)(const char *name, PyObject* (*initfunc)());
 
 LIBPYTHON_EXTERN PyObject* (*Py_BuildValue)(const char *format, ...);
@@ -228,6 +240,7 @@ LIBPYTHON_EXTERN PyObject* (*PyByteArray_FromStringAndSize)(const char *string,
 LIBPYTHON_EXTERN char* (*PyByteArray_AsString)(PyObject *bytearray);
 LIBPYTHON_EXTERN PyObject* (*PyUnicode_FromString)(const char *u);
 
+LIBPYTHON_EXTERN void (*PyErr_Clear)();
 LIBPYTHON_EXTERN void (*PyErr_Fetch)(PyObject **, PyObject **, PyObject **);
 LIBPYTHON_EXTERN PyObject* (*PyErr_Occurred)(void);
 LIBPYTHON_EXTERN void (*PyErr_NormalizeException)(PyObject**, PyObject**, PyObject**);
